--[[
	LevelLogger

	This addon records a log of how long it took to reach each level.
	Use /lvlog to print the current character's log in the chat frame.
	The following parameters are optional:
		/lvlog <number> <level> <channel> <who>
		number = reports the last <number> levels up until <level>, default = 5
			use zero to report all levels logged
		level = reports the log up until this level, default = current level
		channel = indicates a chat channel to report in, default = console output
			s = /say
			y = /yell
			p = /party
			r = /raid
			g = /guild
			w = /whisper
		who = if using channel w, this specifies the whisper target, default = current target
]]--


LevelLogger = LibStub("AceAddon-3.0"):NewAddon( "LevelLogger",
	"AceConsole-3.0", "AceEvent-3.0"
);

-- saved variable containing the level log for each character
svLevelLog = nil;

-- reporting options, saved globally
svLevelLogOptions = {
	showCurrDate = false,
	showCurrTime = false,
	showLevelTime = true,
	showPlayedTime = true,
	reportToParty = true,
	reportToGuild = true,
};

function LevelLogger:OnInitialize()
	-- init workspace vars
	self.newLevel = false;		-- gets set on ding, then used when /played is processed
	self.totalTime = 0;			-- gets set on ding, then used when /played is processed
	self.showCurrent = false;	-- used by "current" command to show time played to current level
	self.startup = true;		-- true between addon loaded and the first /played

	-- init stuff
	self:SetupOptions();
	self:RegisterChatCommand( "lvlog", "OnMsg" );

	-- register events
	self:RegisterEvent("VARIABLES_LOADED");
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("PLAYER_LEVEL_UP");
	self:RegisterEvent("TIME_PLAYED_MSG");
end

function LevelLogger:SetupOptions()
	LibStub("AceConfig-3.0"):RegisterOptionsTable( "LevelLogger", {
		type = "group",
		get = function(info) return svLevelLogOptions[info[#info]]; end,
		set = function(info,val) svLevelLogOptions[info[#info]] = val; end,
		args = {
			showCurrDate = {
				name = "Report Level-Up Date",
				desc = "Report the date when each level is logged.",
				type = "toggle",
				width = "full",
			},
			showCurrTime = {
				name = "Report Level-Up Time",
				desc = "Report the time of day when each level is logged.",
				type = "toggle",
				width = "full",
			},
			showLevelTime = {
				name = "Report Time Played to Level-Up",
				desc = "Report the time /played to reach each level.",
				type = "toggle",
				width = "full",
			},
			showPlayedTime = {
				name = "Report Total Time Played",
				desc = "Report the total time /played when each level is logged.",
				type = "toggle",
				width = "full",
			},
			reportToParty = {
				name = "Send Level-Up Report to Party",
				desc = "Send level up messages to your party's chat channel.",
				type = "toggle",
				width = "full",
			},
			reportToGuild = {
				name = "Send Level-Up Report to Guild",
				desc = "Send level up messages to your guild's chat channel.",
				type = "toggle",
				width = "full",
			},
		}
	} );
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions( "LevelLogger" );
	self.optionsFrame.default = function() self:SetDefaultOptions(); end;
	self.optionsReg = LibStub("AceConfigRegistry-3.0");
end

function LevelLogger:SetDefaultOptions()
	svLevelLogOptions.showCurrDate = false;
	svLevelLogOptions.showCurrTime = false;
	svLevelLogOptions.showLevelTime = true;
	svLevelLogOptions.showPlayedTime = true;
	svLevelLogOptions.reportToParty = true;
	svLevelLogOptions.reportToGuild = true;

	--force config dialog to show new state of options
	self.optionsReg:NotifyChange("LevelLogger");
end

function LevelLogger:ParseTime( strTime )
	--DEFAULT_CHAT_FRAME:AddMessage("LevelLogger: strTime="..strTime);
	local d, h, m, s = string.match( strTime, "(%d+) days (%d+):(%d+):(%d+)" );
	return s + m*60 + h*3600 + d*86400;
end

-- returns logData record formatted into text string
function LevelLogger:LogFormat( logData )
	local strLog = format("Dinged level %d", logData.level);
	if (logData.ts ~= nil) then
		if (svLevelLogOptions.showCurrDate) then		-- include date and time with ding
			strLog = strLog..format( " on %s", date("%d-%b-%Y %H:%M:%S",logData.ts) );
		elseif (svLevelLogOptions.showCurrTime) then	-- include time with ding
			strLog = strLog..format( " at %s", date("%H:%M:%S",logData.ts) );
		end
	end

	if (logData.levelTime > 0 and svLevelLogOptions.showLevelTime and svLevelLogOptions.showPlayedTime) then
		local d1, h1, m1, s1 = ChatFrame_TimeBreakDown(logData.levelTime);
		local d2, h2, m2, s2 = ChatFrame_TimeBreakDown(logData.totalTime);
		strLog = strLog..format(" in %d days %02d:%02d:%02d (of %d days %02d:%02d:%02d total played)",
			d1, h1, m1, s1, d2, h2, m2, s2);
	elseif (logData.levelTime > 0 and svLevelLogOptions.showLevelTime) then		-- include time to level with ding
		local d, h, m, s = ChatFrame_TimeBreakDown(logData.levelTime);
		strLog = strLog..format(" in %d days %02d:%02d:%02d", d, h, m, s);
	elseif (svLevelLogOptions.showPlayedTime) then		-- include /played time with ding
		local d, h, m, s = ChatFrame_TimeBreakDown(logData.totalTime);
		strLog = strLog..format(" at %d days %02d:%02d:%02d total played", d, h, m, s);
	end

	--self:Print("LevelLogger: " .. strLog);
	return strLog..".";
end

function LevelLogger:OnMsg(msg)
	if (msg == "") then
		args = { "" };
	else
		args = { strsplit(" ", msg) };
	end
	if (args[1] == "current") then
		-- calculate and log time to current level (in case it was missed)
		self.showCurrent = true;
		RequestTimePlayed();
	elseif (#svLevelLog == 0) then
		self:Print("No LevelLogger data found for " .. UnitName("player") .. ".");
	else
		local numLevels = tonumber( args[1] );
		local toLevel = tonumber( args[2] );
		local channel = args[3];
		--[[if ( #args > 0 ) then
			numLevels = tonumber( args[0] );
			if ( #args > 1 ) then
				toLevel = tonumber( args[1] );
			end
		end]]
		if (numLevels == nil) then
			numLevels = 5; -- show last 5 levels by default
		elseif (numLevels < 1) then
			numLevels = 999; -- show all levels
		end
		local myLevel = UnitLevel("player");
		if (toLevel == nil or toLevel < 1 or toLevel > myLevel) then
			toLevel = myLevel; -- show most recent levels
		end
		local logFunc;
		if (channel ~= nil) then
			local ch = channel:sub(1,1):lower();
			local map = { s = "SAY", y = "YELL", p = "PARTY", r = "RAID", g = "GUILD", w = "WHISPER" };
			if ( map[ch] ~= nil ) then
				if ( ch ~= "w" ) then
					local okay = true;
					if ( ch == "p" ) then
						okay = GetNumPartyMembers() > 0;
					elseif ( ch == "r" ) then
						okay = GetNumRaidMembers() > 0;
					elseif ( ch == "g" ) then
						okay = IsInGuild();
					end
					if ( okay ) then
						logFunc = function( txt ) SendChatMessage( txt, map[ch] ); end
					end
				else -- get the whisper target
					local w = args[4];
					if ( w == nil or w == "%t" ) then
						w = UnitName("target");
					elseif ( not UnitIsPlayer( w ) ) then
						w = nil;
					end
					if ( w ~= nil ) then
						logFunc = function( txt ) SendChatMessage( txt, "WHISPER", nil, w ); end
					end
				end
			end
		end
		if ( logFunc == nil ) then
			logFunc = function( txt ) self:Print( txt ); end
		end
		-- print out the level log
		logFunc( "LevelLogger report for " .. UnitName("player") .. ":" );
		for i,logData in pairs(svLevelLog) do
			local lvl = tonumber(logData.level);
			if (lvl > toLevel - numLevels and lvl <= toLevel) then
				logFunc( self:LogFormat( logData ) );
			end
		end
	end
end

function LevelLogger:TestDing()
	self.newLevel = 99;
	RequestTimePlayed();
end

function LevelLogger:Ding(lvl,totalTime,levelTime)
	local logData = {
		level = tonumber(lvl);
		levelTime = levelTime;
		totalTime = totalTime;
		ts = time();
	};
	local strLog = self:LogFormat( logData );

	local sentmsg = false;
	if ( logData.levelTime > 0 ) then
		-- append a new level message to the log
		table.insert( svLevelLog, logData );

		if (svLevelLogOptions.reportToParty) then
			if (GetNumPartyMembers() > 0) then
				SendChatMessage( strLog, "PARTY" );
				sentmsg = true;
			end
		end
		if (svLevelLogOptions.reportToGuild) then
			if (IsInGuild()) then
				SendChatMessage( strLog, "GUILD" );
				sentmsg = true;
			end
		end
	end
	if ( not sentmsg ) then
		-- print out the text if not already displayed
		self:Print(strLog);
	end
end

function LevelLogger:VARIABLES_LOADED()
	if (svLevelLog == nil) then
		svLevelLog = {};
	end
end

function LevelLogger:PLAYER_ENTERING_WORLD()
	if self.startup then
		-- after initial load, check if we already got the time played
		if self.totalTime == 0 then
			RequestTimePlayed();
		end
		self.startup = false;
	else
		-- always update the time played when zoning
		RequestTimePlayed();
	end
end

function LevelLogger:PLAYER_LEVEL_UP(eventname, theLevel)
	self.newLevel = theLevel;
	RequestTimePlayed();
end

function LevelLogger:TIME_PLAYED_MSG(eventname, timePlayedTotal, timePlayedLevel)
	if (self.newLevel) then
		-- timePlayedLevel should be zero, so we need to calculate the level up time
		-- use last known times and add the difference from current total played time
		local levelTime = self.levelTime + ( timePlayedTotal - self.totalTime );
		self:Ding(self.newLevel,timePlayedTotal,levelTime);
		self.newLevel = false;
	elseif (self.showCurrent) then
		self:Ding(UnitLevel("player"), timePlayedTotal - timePlayedLevel, 0);
		self.showCurrent = false;
	end
	self.totalTime = timePlayedTotal;
	self.levelTime = timePlayedLevel;
end
